==> bettertrims-4.0.0%2B1.21.1-fabric-sources.jar: com/bawnorton/bettertrims/property/ability/type/entity/ExplodeAbility.java <==
package com.bawnorton.bettertrims.property.ability.type.entity;

import com.bawnorton.bettertrims.client.tooltip.component.CompositeContainerComponent;
import com.bawnorton.bettertrims.client.tooltip.element.TrimElementTooltipProvider;
import com.bawnorton.bettertrims.property.ability.type.TrimEntityAbility;
import com.bawnorton.bettertrims.property.context.TrimmedItems;
import com.bawnorton.bettertrims.property.count.CountBasedValue;
import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import org.jetbrains.annotations.Nullable;

import java.util.Optional;
import net.minecraft.class_124;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1304;
import net.minecraft.class_1309;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2394;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_3218;
import net.minecraft.class_3414;
import net.minecraft.class_5684;
import net.minecraft.class_638;
import net.minecraft.class_6880;
import net.minecraft.class_6885;
import net.minecraft.class_6895;
import net.minecraft.class_7924;
import net.minecraft.class_8110;
import net.minecraft.class_9749;

public record ExplodeAbility(
		boolean attributeToWearer,
		Optional<class_6880<class_8110>> damageType,
		Optional<CountBasedValue> knockbackMultiplier,
		Optional<class_6885<class_2248>> immuneBlocks,
		class_243 offset,
		CountBasedValue radius,
		boolean createFire,
		class_1937.class_7867 blockInteraction,
		class_2394 smallParticle,
		class_2394 largeParticle,
		class_6880<class_3414> sound
) implements TrimEntityAbility {
	public static final MapCodec<ExplodeAbility> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
			Codec.BOOL.optionalFieldOf("attribute_to_wearer", false).forGetter(ExplodeAbility::attributeToWearer),
			class_8110.field_51565.optionalFieldOf("damage_type").forGetter(ExplodeAbility::damageType),
			CountBasedValue.CODEC.optionalFieldOf("knockback_multiplier").forGetter(ExplodeAbility::knockbackMultiplier),
			class_6895.method_40340(class_7924.field_41254).optionalFieldOf("immune_blocks").forGetter(ExplodeAbility::immuneBlocks),
			class_243.field_38277.optionalFieldOf("offset", class_243.field_1353).forGetter(ExplodeAbility::offset),
			CountBasedValue.CODEC.fieldOf("radius").forGetter(ExplodeAbility::radius),
			Codec.BOOL.optionalFieldOf("create_fire", false).forGetter(ExplodeAbility::createFire),
			class_1937.class_7867.field_51780.fieldOf("block_interaction").forGetter(ExplodeAbility::blockInteraction),
			class_2398.field_25125.fieldOf("small_particle").forGetter(ExplodeAbility::smallParticle),
			class_2398.field_25125.fieldOf("large_particle").forGetter(ExplodeAbility::largeParticle),
			class_3414.field_41699.fieldOf("sound").forGetter(ExplodeAbility::sound)
	).apply(instance, ExplodeAbility::new));

	@Override
	public void apply(class_3218 level, class_1309 wearer, class_1297 target, TrimmedItems items, @Nullable class_1304 targetSlot, class_243 origin) {
		int count = items.size();
		level.method_8454(
				attributeToWearer ? wearer : null,
				getDamageSource(wearer, origin),
				new class_9749(
						blockInteraction != class_1937.class_7867.field_40888,
						damageType.isPresent(),
						knockbackMultiplier.map(countBasedValue -> countBasedValue.calculate(count)),
						immuneBlocks
				),
				origin.method_10216(),
				origin.method_10214(),
				origin.method_10215(),
				Math.max(radius.calculate(count), 0F),
				createFire,
				blockInteraction,
				smallParticle,
				largeParticle,
				sound
		);
	}

	private class_1282 getDamageSource(class_1297 entity, class_243 pos) {
		return this.damageType.map(damageTypeHolder -> this.attributeToWearer ? new class_1282(damageTypeHolder, entity) : new class_1282(damageTypeHolder, pos)).orElse(null);
	}

	@Override
	public boolean usesCount() {
		return true;
	}

	@Override
	public MapCodec<? extends TrimEntityAbility> codec() {
		return CODEC;
	}

	public static class TooltipProvider implements TrimElementTooltipProvider<ExplodeAbility> {
		@Override
		public class_5684 getTooltip(class_638 level, ExplodeAbility element, boolean includeCount) {
			return CompositeContainerComponent.builder()
					.translate("bettertrims.tooltip.ability.explode.explodes_with_radius", style -> style.method_10977(class_124.field_1079))
					.cycle(builder -> element.radius().getValueComponents(4, includeCount).forEach(builder::textComponent))
					.spaced()
					.build();
		}
	}
}

==> bettertrims-4.0.2%2B1.21.10-fabric-sources.jar: com/bawnorton/bettertrims/property/ability/type/entity/ExplodeAbility.java <==
package com.bawnorton.bettertrims.property.ability.type.entity;

import com.bawnorton.bettertrims.client.tooltip.component.CompositeContainerComponent;
import com.bawnorton.bettertrims.client.tooltip.element.TrimElementTooltipProvider;
import com.bawnorton.bettertrims.property.ability.type.TrimEntityAbility;
import com.bawnorton.bettertrims.property.context.TrimmedItems;
import com.bawnorton.bettertrims.property.count.CountBasedValue;
import com.mojang.serialization.Codec;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import org.jetbrains.annotations.Nullable;

import java.util.Optional;
import net.minecraft.class_11749;
import net.minecraft.class_124;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1304;
import net.minecraft.class_1309;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2394;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_3218;
import net.minecraft.class_3414;
import net.minecraft.class_5684;
import net.minecraft.class_6012;
import net.minecraft.class_638;
import net.minecraft.class_6880;
import net.minecraft.class_6885;
import net.minecraft.class_6895;
import net.minecraft.class_7924;
import net.minecraft.class_8110;
import net.minecraft.class_9749;

public record ExplodeAbility(
		boolean attributeToWearer,
		Optional<class_6880<class_8110>> damageType,
		Optional<CountBasedValue> knockbackMultiplier,
		Optional<class_6885<class_2248>> immuneBlocks,
		class_243 offset,
		CountBasedValue radius,
		boolean createFire,
		class_1937.class_7867 blockInteraction,
		class_2394 smallParticle,
		class_2394 largeParticle,
		//? if >=1.21.10 {
		class_6012<class_11749> blockParticles,
		//?}
		class_6880<class_3414> sound
) implements TrimEntityAbility {
	public static final MapCodec<ExplodeAbility> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
			Codec.BOOL.optionalFieldOf("attribute_to_wearer", false).forGetter(ExplodeAbility::attributeToWearer),
			class_8110.field_51565.optionalFieldOf("damage_type").forGetter(ExplodeAbility::damageType),
			CountBasedValue.CODEC.optionalFieldOf("knockback_multiplier").forGetter(ExplodeAbility::knockbackMultiplier),
			class_6895.method_40340(class_7924.field_41254).optionalFieldOf("immune_blocks").forGetter(ExplodeAbility::immuneBlocks),
			class_243.field_38277.optionalFieldOf("offset", class_243.field_1353).forGetter(ExplodeAbility::offset),
			CountBasedValue.CODEC.fieldOf("radius").forGetter(ExplodeAbility::radius),
			Codec.BOOL.optionalFieldOf("create_fire", false).forGetter(ExplodeAbility::createFire),
			class_1937.class_7867.field_51780.fieldOf("block_interaction").forGetter(ExplodeAbility::blockInteraction),
			class_2398.field_25125.fieldOf("small_particle").forGetter(ExplodeAbility::smallParticle),
			class_2398.field_25125.fieldOf("large_particle").forGetter(ExplodeAbility::largeParticle),
			//? if >=1.21.10 {
			class_6012.method_66213(class_11749.field_62054).optionalFieldOf("block_particles", class_6012.method_34990()).forGetter(ExplodeAbility::blockParticles),
			//?}
			class_3414.field_41699.fieldOf("sound").forGetter(ExplodeAbility::sound)
	).apply(instance, ExplodeAbility::new));

	@Override
	public void apply(class_3218 level, class_1309 wearer, class_1297 target, TrimmedItems items, @Nullable class_1304 targetSlot, class_243 origin) {
		int count = items.size();
		level.method_8454(
				attributeToWearer ? wearer : null,
				getDamageSource(wearer, origin),
				new class_9749(
						blockInteraction != class_1937.class_7867.field_40888,
						damageType.isPresent(),
						knockbackMultiplier.map(countBasedValue -> countBasedValue.calculate(count)),
						immuneBlocks
				),
				origin.method_10216(),
				origin.method_10214(),
				origin.method_10215(),
				Math.max(radius.calculate(count), 0F),
				createFire,
				blockInteraction,
				smallParticle,
				largeParticle,
				//? if >=1.21.10 {
				blockParticles,
				//?}
				sound
		);
	}

	private class_1282 getDamageSource(class_1297 entity, class_243 pos) {
		return this.damageType.map(damageTypeHolder -> this.attributeToWearer ? new class_1282(damageTypeHolder, entity) : new class_1282(damageTypeHolder, pos)).orElse(null);
	}

	@Override
	public boolean usesCount() {
		return true;
	}

	@Override
	public MapCodec<? extends TrimEntityAbility> codec() {
		return CODEC;
	}

	public static class TooltipProvider implements TrimElementTooltipProvider<ExplodeAbility> {
		@Override
		public class_5684 getTooltip(class_638 level, ExplodeAbility element, boolean includeCount) {
			return CompositeContainerComponent.builder()
					.translate("bettertrims.tooltip.ability.explode.explodes_with_radius", style -> style.method_10977(class_124.field_1079))
					.cycle(builder -> element.radius().getValueComponents(4, includeCount).forEach(builder::textComponent))
					.spaced()
					.build();
		}
	}
}
